/***************************************************************************
 * Release Date	: 9/8/2001
 * Author		: Paul Rosen ([BwC]ScatMan)
 * Email		: rosen@purdue.edu
 * Website		: http://fullspeed.to/cstrike/
 *
 * plugin_stats.sma
 * version 0.2
 ***************************************************************************/

#include <core>
#include <console>
#include <string>
#include <admin>
#include <adminlib>

#define PRINT_PRIVATE 1;
#define PRINT_PUBLIC 2;
#define PRINT_PUBLIC_NO_HEADER 3;
#define PRINT_PUBLIC_NO_BODY 4;

new DEFAULT_MM = PRINT_PUBLIC;

new WAIT_TIME = 5;
new MM_OWN_STATS = PRINT_PUBLIC;
new MM_OTHERS_STATS = PRINT_PUBLIC;
new MM_FIND = PRINT_PUBLIC;

new SAY_NAME[MAX_NAME_LENGTH];
new SAY_TEXT[MAX_TEXT_LENGTH];
new LAST_SAY_TIME[MAX_PLAYERS];

new STRING_VERSION[MAX_DATA_LENGTH] = "2.50.e";

new FILENAME[MAX_TEXT_LENGTH] = "stats.dat";
new TEMPLATE[MAX_TEXT_LENGTH] = "plugin_stats.txt";

public plugin_init() {
	plugin_registerinfo("ScatMan's PsychoStats Stat Plugin","Gives players their current stats.",STRING_VERSION);
	
	plugin_registercmd("say","handle_say",ACCESS_ALL);
	plugin_registercmd("admin_stats_reload", "admin_stats_reload", ACCESS_RCON, "admin_stats_reload:  Reloads the stats config file.");

	plugin_registerhelp("say",ACCESS_ALL,"say /stats: Get your current stats, if you are ranked.");
	plugin_registerhelp("say",ACCESS_ALL,"say /stats help: Gives you complete usage on the stats plugin.");

	read_settings();
	
	return PLUGIN_CONTINUE;
}

public plugin_connect(HLUserName, HLIP, UserIndex) 
{
	if (UserIndex >= 1 && UserIndex <= MAX_PLAYERS){
		LAST_SAY_TIME[UserIndex] = 0;
	}

	return PLUGIN_CONTINUE;
}


/****************************************************************************\
|****************************************************************************|
\****************************************************************************/
public admin_stats_reload(HLCommand, HLData, HLUserName, UserIndex){
	read_settings();
	selfmessage("Success Reading Stats Settings!");
	return PLUGIN_HANDLED;
}

public print_text(mmode){
	if(mmode == PRINT_PRIVATE){
		messageex(SAY_NAME, SAY_TEXT, print_chat);
	}
	else if(mmode == PRINT_PUBLIC){
		say(SAY_TEXT);
	}
}

public handle_say(HLCommand,HLData,HLUserName,UserIndex) {
	new Data[MAX_DATA_LENGTH];
	new Text[MAX_TEXT_LENGTH];
	
	convert_string(HLData,Data,MAX_DATA_LENGTH);
	convert_string(HLUserName,SAY_NAME,MAX_NAME_LENGTH);

	strstripquotes(Data);

	new lineCommand1[50];
	new lineCommand2[50];
	new plyrName[50];

	strsplit(Data, " ", lineCommand1, 50 , lineCommand2, 50 , plyrName, 50);

	if (streq(Data, "/stats")==1 || streq(Data, "show my stats")==1) {
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "%s :    %s", SAY_NAME, Data);	// Set global SAY_TEXT to the user's say command

		if(systemtime()  >= LAST_SAY_TIME[UserIndex] + WAIT_TIME){

			if(MM_OWN_STATS == PRINT_PUBLIC || MM_OWN_STATS == PRINT_PUBLIC_NO_BODY){
				print_text(PRINT_PUBLIC);		// Print out the user's say command
			}
			else{						// Public or private depending upon the server's settings
				print_text(PRINT_PRIVATE);
			}

			scat_stats();					// Get stats and print them out

			LAST_SAY_TIME[UserIndex] = systemtime();	// update the time for our simple flood control

			return PLUGIN_HANDLED;			// return plugin_handled
		}
		else{
			// The following gets run if the user's command comes before his flood time has passed
			print_text(PRINT_PRIVATE);		// SAY_TEXT set to the "player_name:    /stats" so print it out in private mode

			numtostr(WAIT_TIME, Text);	// Make wait_time something that we can print out.  Inform user that they must wait
			snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "Sorry, the server admin has set a %s second limit between stats commands.", Text);
			print_text(PRINT_PRIVATE);	// Print out SAY_TEXT in private mode
		
			return PLUGIN_HANDLED;
		}
	}
	else if (streq(Data, "/stats help")==1) {
		// '/stats help' command ALWAYS gets printed publicly
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "%s :    %s", SAY_NAME, Data);
		print_text(PRINT_PUBLIC);
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "Usage on the /stats command");
		print_text(PRINT_PUBLIC);
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "/stats: returns your current rank and stats");
		print_text(PRINT_PUBLIC);
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "/stats find [string]: returns list of players matching [string]");
		print_text(PRINT_PUBLIC);
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "/stats [rank]: returns the stats on the player with [rank] as their current rank");
		print_text(PRINT_PUBLIC);
		return PLUGIN_HANDLED;
	}
	else if (streq(Data, "/stats about")==1) {
		// '/stats about' command ALWAYS gets printed publicly
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "%s :    %s", SAY_NAME, Data);
		print_text(PRINT_PUBLIC);
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "Psycho Server Stats, PsychoStats AdminMod Plugin v0.1");
		print_text(PRINT_PUBLIC);
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "Developed by [BwC]ScatMan");
		print_text(PRINT_PUBLIC);
		return PLUGIN_HANDLED;
	}
	else if (streq(lineCommand1, "/stats")==1 && streq(lineCommand2, "find")==1) {
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "%s :    %s", SAY_NAME, Data);
		if(MM_FIND == PRINT_PUBLIC || MM_FIND == PRINT_PUBLIC_NO_BODY){
			print_text(PRINT_PUBLIC);		// Print out the user's say command
		}
		else{						// Public or private depending upon the server's settings
			print_text(PRINT_PRIVATE);
		}

		if(strlen(plyrName) < 3){
			strcpy(SAY_TEXT, "Find String Must Be Atleast 3 Characters Long.", MAX_TEXT_LENGTH);
			if(MM_FIND == PRINT_PUBLIC || MM_FIND == PRINT_PUBLIC_NO_BODY){
				print_text(PRINT_PUBLIC);		// Print out the user's say command
			}
			else{						// Public or private depending upon the server's settings
				print_text(PRINT_PRIVATE);
			}
		}
		else{
			find_stats(plyrName);
		}
		return PLUGIN_HANDLED;
	}
	else if (streq(lineCommand1, "/stats")==1){
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "%s :    %s", SAY_NAME, Data);
		if(systemtime()  >= LAST_SAY_TIME[UserIndex] + WAIT_TIME){
			if(MM_OTHERS_STATS == PRINT_PUBLIC || MM_OTHERS_STATS == PRINT_PUBLIC_NO_BODY){
				print_text(PRINT_PUBLIC);		// Print out the user's say command
			}
			else{						// Public or private depending upon the server's settings
				print_text(PRINT_PRIVATE);
			}

			getName_stats(lineCommand2);
			LAST_SAY_TIME[UserIndex] = systemtime();
			return PLUGIN_HANDLED;
		}
		numtostr(WAIT_TIME, Text);
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "Sorry, the server admin has set a %s second limit between stats commands.", Text);
		print_text(PRINT_PRIVATE);	// Print out SAY_TEXT in private mode
		return PLUGIN_HANDLED;
	}

	return PLUGIN_CONTINUE;
}	


public find_stats(UserName[])
{
	new iLineNum = 0;
	new strLineBuf[MAX_TEXT_LENGTH];
	new playerFound = 0;

	if (fileexists(FILENAME) > 0){
		if(!readfile(FILENAME, strLineBuf, iLineNum, MAX_TEXT_LENGTH)){
			return makeReadable();
		}

		do{
			new tmpName[50];
			strsplit(strLineBuf, "%", tmpName, 50);

			if(strcasestr(tmpName, UserName) >= 0){
				new plyrName[50];
				new plyrRank[50];
	
				strsplit(strLineBuf, "%", plyrName, 50, plyrRank, 50);
				snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "Rank %s: %s", plyrRank, plyrName);
				if(MM_FIND == PRINT_PUBLIC || MM_FIND == PRINT_PUBLIC_NO_HEADER){
					print_text(PRINT_PUBLIC);
				}
				else{
					print_text(PRINT_PRIVATE);
				}
				playerFound = 1;
			}
		} while(readfile(FILENAME, strLineBuf, ++iLineNum, MAX_TEXT_LENGTH));

		if(playerFound == 0){
			snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "No Players Found");
			if(MM_FIND == PRINT_PUBLIC || MM_FIND == PRINT_PUBLIC_NO_HEADER){
				print_text(PRINT_PUBLIC);
			}
			else{
				print_text(PRINT_PRIVATE);
			}
		}
	}
	else
	{
		new OutText[MAX_TEXT_LENGTH];
		snprintf(OutText, MAX_TEXT_LENGTH, "PsychoServer Plugin - Cannot find %s file.", FILENAME);
		log(OutText);
		return 1;
	}
	return 0;
}

public makeReadable()
{
	exec("file_access_read 1");
	return 0;
}

public getName_stats(Rank[])
{
	new iLineNum = 0;
	new strLineBuf[MAX_TEXT_LENGTH];
	new playerFound = 0;

	if (fileexists(FILENAME) > 0)
	{

		if (!readfile(FILENAME, strLineBuf, iLineNum, MAX_TEXT_LENGTH)){
			return makeReadable();
		}

		do
		{
			new tmpName[50];
			new tmpRank[50];
			strsplit(strLineBuf, "%", tmpName, 50, tmpRank, 50);

			if(strcmp(tmpRank, Rank) == 0){
				playerFound = 1;
				if(MM_OTHERS_STATS == PRINT_PUBLIC || MM_OTHERS_STATS == PRINT_PUBLIC_NO_HEADER){
					output_player(strLineBuf, PRINT_PUBLIC);
				}
				else{
					output_player(strLineBuf, PRINT_PRIVATE);
				}

			}
		}while (playerFound == 0 && readfile(FILENAME, strLineBuf, ++iLineNum, MAX_TEXT_LENGTH))

		if(playerFound == 0){
			snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "%s not found on this server.", Rank);
			if(MM_FIND == PRINT_PUBLIC || MM_FIND == PRINT_PUBLIC_NO_HEADER){
				print_text(PRINT_PUBLIC);
			}
			else{
				print_text(PRINT_PRIVATE);
			}
		}
	}
	else
	{
		new OutText[MAX_TEXT_LENGTH];
		snprintf(OutText, MAX_TEXT_LENGTH, "PsychoServer Plugin - Cannot find %s file.", FILENAME);
		log(OutText);
		return 1;
	}

	return 0;
}

public output_player(fileline[], mm)
{
	new plyrName[50];
	new plyrRank[50];
	new plyrKills[50];
	new plyrDeaths[50];
	new plyrKdr[50];
	new plyrKpm[50];
	new plyrSkill[50];

	new iLineNum = 0;
	new strLineBuf[MAX_TEXT_LENGTH];
	new iSubs = 0;
	new i = 0;
	new outputNow = 0;

	strsplit(fileline, "%", plyrName, 50, plyrRank, 50, plyrKills, 50, plyrDeaths, 50, plyrKdr, 50, plyrKpm, 50, plyrSkill, MAX_TEXT_LENGTH);

	if (fileexists(TEMPLATE) > 0)
	{
		if (!readfile(TEMPLATE, strLineBuf, iLineNum, MAX_TEXT_LENGTH)){
			return makeReadable();
		}

		do{

			if(strncmp(strLineBuf, "}", 1) == 0){
				outputNow = 0;
			}
			else if (((strncmp(strLineBuf, "#", 1) != 0) && (strncmp(strLineBuf, "//", 2) != 0)) && (outputNow == 1))
			{
				iSubs = strcount(strLineBuf, '%') / 2;
				for (i = 0; i < iSubs; i++)
				{
					strsubst(strLineBuf, "%NAME%", plyrName, 50);
					strsubst(strLineBuf, "%SKILL%", plyrSkill, 50);
					strsubst(strLineBuf, "%RANK%", plyrRank, 50);
					strsubst(strLineBuf, "%KILLS%", plyrKills, 50);
					strsubst(strLineBuf, "%DEATHS%", plyrDeaths, 50);
					strsubst(strLineBuf, "%KPM%", plyrKdr, 50);
					strsubst(strLineBuf, "%KDR%", plyrKpm, 50);
				}

				strcpy(SAY_TEXT, strLineBuf, MAX_TEXT_LENGTH);
				print_text(mm);
			}
			else if(strncmp(strLineBuf, "command_stats{", 14) == 0){
				outputNow = 1;
			}

		} while(readfile(TEMPLATE, strLineBuf, ++iLineNum, MAX_TEXT_LENGTH));
	}
	else{
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "Current Stats for Player: %s", plyrName);
		print_text(mm);
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "Rank: %s      Skill: %s      KPM: %s", plyrRank, plyrSkill, plyrKpm);
		print_text(mm);
		snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "Kills: %s    Deaths: %s      KDR: %s", plyrKills, plyrDeaths, plyrKdr);
		print_text(mm);
	}

	return 0;
}

public scat_stats()
{
	new iLineNum = 0;
	new strLineBuf[MAX_TEXT_LENGTH];
	new playerFound = 0;


	if (fileexists(FILENAME) > 0)
	{
		if (!readfile(FILENAME, strLineBuf, iLineNum, MAX_TEXT_LENGTH)){
			return makeReadable();
		}

		do{
			new tmpName[50];
			strsplit(strLineBuf, "%", tmpName, 50);

			if(strcasecmp(tmpName, SAY_NAME) == 0){
				playerFound = 1;

				if(MM_OWN_STATS == PRINT_PUBLIC || MM_OWN_STATS == PRINT_PUBLIC_NO_HEADER){
					output_player(strLineBuf, PRINT_PUBLIC);
				}
				else{
					output_player(strLineBuf, PRINT_PRIVATE);
				}
			}
		}while (playerFound == 0 && readfile(FILENAME, strLineBuf, ++iLineNum, MAX_TEXT_LENGTH))


		if(playerFound == 0){
			snprintf(SAY_TEXT, MAX_TEXT_LENGTH, "%s is currently not ranked on this server.", SAY_NAME);
			if(MM_OWN_STATS == PRINT_PUBLIC || MM_OWN_STATS == PRINT_PUBLIC_NO_HEADER){
				print_text(PRINT_PUBLIC);
			}
			else{
				print_text(PRINT_PRIVATE);
			}
		}

	}
	else
	{
		new OutText[MAX_TEXT_LENGTH];
		snprintf(OutText, MAX_TEXT_LENGTH, "PsychoServer Plugin - Cannot find %s file.", FILENAME);
		log(OutText);
		return 1;
	}

	return 0;
}


public read_settings(){
	new iLineNum = 0;
	new strLineBuf[MAX_TEXT_LENGTH];
	//new iSubs = 0;
	//new i = 0;
	new readSettings = 0;
	new Text[MAX_TEXT_LENGTH];

	if (fileexists(TEMPLATE) > 0)
	{
		if (!readfile(TEMPLATE, strLineBuf, iLineNum, MAX_TEXT_LENGTH)){
			return makeReadable();
		}

		do{
			if(strncmp(strLineBuf, "}", 1) == 0){
				readSettings = 0;
			}
			else if (((strncmp(strLineBuf, "#", 1) != 0) && (strncmp(strLineBuf, "//", 2) != 0)) && (readSettings == 1))
			{
				new setting[50];
				new value[50];
				strsplit(strLineBuf, " =;^t^n", setting, 50 , value, 50);

				if(strcasecmp(setting, "MESSAGE_MODE{OWN_STATS}") == 0){
					MM_OWN_STATS = get_MM(value);
				}
				else if(strcasecmp(setting, "MESSAGE_MODE{OTHERS_STATS}") == 0){
					MM_OTHERS_STATS = get_MM(value);
				}
				else if(strcasecmp(setting, "MESSAGE_MODE{FIND}") == 0){
					MM_FIND = get_MM(value);
				}
				else if(strcasecmp(setting, "WAIT_TIME") == 0){
					if(strcasecmp(value, "OFF") == 0){
						WAIT_TIME = -1;
					}
					else if(strcasecmp(value, "FIVE") == 0){
						WAIT_TIME = 5;
					}
					else if(strcasecmp(value, "TEN") == 0){
						WAIT_TIME = 10;
					}
					else if(strcasecmp(value, "FIFTEEN") == 0){
						WAIT_TIME = 15;
					}
					else{
						snprintf(Text, MAX_TEXT_LENGTH, "plugin_stats.amx - Unrecognized Wait Time: %s", value);
					}
				}
				else if(strcasecmp(setting, "STATS_FILE") == 0){
					strcpy(FILENAME, value, MAX_TEXT_LENGTH);
				}
			}
			else if(strncmp(strLineBuf, "settings{", 9) == 0){
				readSettings = 1;
			}

		} while(readfile(TEMPLATE, strLineBuf, ++iLineNum, MAX_TEXT_LENGTH));
	}
	return 0;
}

public get_MM(mm[]){
	if(strcasecmp(mm, "PRINT_PRIVATE") == 0){
		return PRINT_PRIVATE;
	}
	if(strcasecmp(mm, "PRINT_PUBLIC") == 0){
		return PRINT_PUBLIC;
	}
	if(strcasecmp(mm, "PRINT_PUBLIC_NO_BODY") == 0){
		return PRINT_PUBLIC_NO_BODY;
	}
	if(strcasecmp(mm, "PRINT_PRIVATE") == 0){
		return PRINT_PUBLIC_NO_HEADER;
	}
	new Text[MAX_TEXT_LENGTH];
	snprintf(Text, MAX_TEXT_LENGTH, "plugin_stats.amx - Unrecognized Message Mode: %s", mm);
	log(Text);
	return DEFAULT_MM;
}